@@ -4,7 +4,7 @@ module test_manifest
     use testsuite, only : new_unittest, unittest_t, error_t, test_failed, check_string
     use fpm_manifest
     use fpm_manifest_profile, only: profile_config_t, find_profile
-    use fpm_strings, only: operator(.in.)
+    use fpm_strings, only: operator(.in.), string_t
     use fpm_error, only: fatal_error, error_t
     implicit none
     private
@@ -46,6 +46,7 @@ contains
             & new_unittest("build-key-invalid", test_build_invalid_key), &
             & new_unittest("library-empty", test_library_empty), &
             & new_unittest("library-wrongkey", test_library_wrongkey, should_fail=.true.), &
+            & new_unittest("library-list", test_library_list, should_fail=.true.), &
             & new_unittest("package-simple", test_package_simple), &
             & new_unittest("package-empty", test_package_empty, should_fail=.true.), &
             & new_unittest("package-typeerror", test_package_typeerror, should_fail=.true.), &
@@ -167,6 +168,10 @@ contains
             return
         end if
 
+        ! Test package serialization
+        call package%test_serialization('test_valid_manifest',error)
+        if (allocated(error)) return
+
     end subroutine test_valid_manifest
 
 
@@ -220,6 +225,9 @@ contains
             return
         end if
 
+        call package%test_serialization('test_default_library',error)
+        if (allocated(error)) return
+
     end subroutine test_default_library
 
 
@@ -243,6 +251,9 @@ contains
             & "Default executable name")
         if (allocated(error)) return
 
+        call package%test_serialization('test_default_executable',error)
+        if (allocated(error)) return
+
     end subroutine test_default_executable
 
 
@@ -517,6 +528,9 @@ contains
             return
         end if
 
+        call chosen_profile%test_serialization('profile serialization: '//profile_name//' '//compiler,error)
+        if (allocated(error)) return
+
         profile_name = 'release'
         compiler = 'gfortran'
         call find_profile(package%profiles, profile_name, compiler, 3, profile_found, chosen_profile)
@@ -525,6 +539,9 @@ contains
             return
         end if
 
+        call chosen_profile%test_serialization('profile serialization: '//profile_name//' '//compiler,error)
+        if (allocated(error)) return
+
         profile_name = 'publish'
         compiler = 'gfortran'
         call find_profile(package%profiles, profile_name, compiler, 1, profile_found, chosen_profile)
@@ -533,15 +550,21 @@ contains
             return
         end if
 
+        call chosen_profile%test_serialization('profile serialization: '//profile_name//' '//compiler,error)
+        if (allocated(error)) return
+
         profile_name = 'debug'
         compiler = 'ifort'
         call find_profile(package%profiles, profile_name, compiler, 3, profile_found, chosen_profile)
         if (.not.(chosen_profile%flags.eq.&
-            ' /warn:all /check:all /error-limit:1 /Od /Z7 /assume:byterecl /standard-semantics /traceback')) then
+            ' /warn:all /check:all /error-limit:1 /Od /Z7 /assume:byterecl /traceback')) then
             call test_failed(error, "Failed to load built-in profile "//profile_name)
             return
         end if
 
+        call chosen_profile%test_serialization('profile serialization: '//profile_name//' '//compiler,error)
+        if (allocated(error)) return
+
         profile_name = 'release'
         compiler = 'ifort'
         call find_profile(package%profiles, profile_name, compiler, 1, profile_found, chosen_profile)
@@ -549,6 +572,10 @@ contains
             call test_failed(error, "Failed to overwrite built-in profile")
             return
         end if
+
+        call chosen_profile%test_serialization('profile serialization: '//profile_name//' '//compiler,error)
+        if (allocated(error)) return
+
     end subroutine test_profiles
 
     !> 'flags' is a key-value entry, test should fail as it is defined as a table
@@ -861,6 +888,47 @@ contains
 
     end subroutine test_library_wrongkey
 
+   !> Pass a TOML table with not allowed source dirs
+    subroutine test_library_list(error)
+        use fpm_manifest_library
+        use fpm_toml, only : new_table, set_list, toml_table
+
+        !> Error handling
+        type(error_t), allocatable, intent(out) :: error
+
+        type(string_t), allocatable :: source_dirs(:)
+        type(toml_table) :: table
+        type(library_config_t) :: library
+
+        source_dirs = [string_t("src1"),string_t("src2")]
+        call new_table  (table)
+        call set_list   (table, "source-dir", source_dirs, error)
+        call new_library(library, table, error)
+
+    end subroutine test_library_list
+
+    !> Pass a TOML table with a 1-sized source dir list
+    subroutine test_library_listone(error)
+        use fpm_manifest_library
+        use fpm_toml, only : new_table, set_list, toml_table
+
+        !> Error handling
+        type(error_t), allocatable, intent(out) :: error
+
+        type(package_config_t) :: package
+        character(:), allocatable :: temp_file
+        integer :: unit
+
+        open(file=temp_file, newunit=unit)
+        write(unit, '(a)') &
+            & 'name = "example"', &
+            & '[library]', &
+            & 'source-dir = ["my-src"]'
+        close(unit)
+
+        call get_package_data(package, temp_file, error)
+
+    end subroutine test_library_listone
 
     !> Packages cannot be created from empty tables
     subroutine test_package_simple(error)
@@ -1207,6 +1275,11 @@ contains
         call set_value(table, "link", "z", stat=stat)
 
         call new_build_config(build, table, 'test_link_string', error)
+        if (allocated(error)) return
+
+        !> Test serialization roundtrip
+        call build%test_serialization('test_link_string', error)
+        if (allocated(error)) return
 
     end subroutine test_link_string
 
@@ -1230,6 +1303,11 @@ contains
         call set_value(children, 2, "lapack", stat=stat)
 
         call new_build_config(build, table, 'test_link_array', error)
+        if (allocated(error)) return
+
+        !> Test serialization roundtrip
+        call build%test_serialization('test_link_array', error)
+        if (allocated(error)) return
 
     end subroutine test_link_array
 
